
; ATMega8

; 07.02.2009 M'Soft Berlin

.equ	VER		= '0'
.equ	REV		= '1'

.include "m8def.inc"

.include "Definitionen.inc"

;****************************** Verktor-Liste ************************

.CSEG
       rjmp    Reset
			 
;****************************** Programm-Beginn **********************

;*********************************************************************
;  Initialisierungsroutine
;*********************************************************************

.ORG	INT_VECTORS_SIZE

; Hardware initialisieren

Reset:
		ldi     TEMP_0,low(RAMEND)          ; Stack an das interne Ram-Ende
        out     SPL,TEMP_0
		ldi     TEMP_0,high(RAMEND)          ; Stack an das interne Ram-Ende
        out     SPH,TEMP_0

; Port B *************************************************************

		ser		TEMP_0						; PullUp ein, Ausgnge H			
        out     PORTB,TEMP_0 
		clr		TEMP_0 
		out     DDRB,TEMP_0

; Port C *************************************************************

		ldi		TEMP_0,(1<<RFM12_CS)		
        out     PORTC,TEMP_0 
		ldi		TEMP_0,(1<<RFM12_SDI) | (1<<RFM12_SCK) | (1<<RFM12_CS) 
		out     DDRC,TEMP_0

; Port D *************************************************************

		ser		TEMP_0						; PullUp ein, Ausgnge H			
        out     PORTD,TEMP_0 
		clr		TEMP_0						; alles Eingang 
		out     DDRD,TEMP_0

; UART ***************************************************************

        ldi     TEMP_0,high(UART_BAUD_SELECT)
        out     UBRRH,TEMP_0
        ldi     TEMP_0,low(UART_BAUD_SELECT)
        out     UBRRL,TEMP_0
        ldi     TEMP_0,(1<<TXEN) | (1<<RXEN)
        out     UCSRB,TEMP_0
        ldi     TEMP_0,(1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0)
        out     UCSRC,TEMP_0

;*********************************************************************				 

; Software initialisieren

		ldi		TEMP_0,250				; 200ms

init_wait_loop:
		rcall	wait_20ms
		dec		TEMP_0
		brne	init_wait_loop

		rcall	uart_send_newline
		load_p	Z,(2*text_reset)
		rcall	uart_send_text

		rcall	rfm12_init

		rcall	uart_send_newline
		load_p	Z,(2*text_rfm_init)
		rcall	uart_send_text
											 
;*********************************************************************
;  Main-Loop
;*********************************************************************

main:	
		ldi		TEMP_0,18
		rcall	rfm12_read_fifo

		load_p	Y,DATA_BUF
		ld		TEMP_0,Y+
		ld		TEMP_1,Y+
		com		TEMP_1
		cp		TEMP_0,TEMP_1			; Sensornummer ok?
		brne	main					; nein

		load_p	Z,DATA_BUF
		ldi		TEMP_1,17
		clr		TEMP_2

main_loop:
		ld		TEMP_0,Z+
		add		TEMP_2,TEMP_0
		dec		TEMP_1
		brne	main_loop

		ld		TEMP_0,Z
		cp		TEMP_0,TEMP_2			; Prfsumme ok?
		brne	main					; nein

		load_p	Z,(2*text_sensor)
		rcall	uart_send_text

		lds		TEMP_0,DATA_BUF
		rcall	uart_send_byte

		ldi		TEMP_0,':'
		rcall	uart_send_byte

		lds		TEMP_0,DATA_BUF+2		; Beginn Text
		cpi		TEMP_0,'V'				; Version?
		brne	main_daten

		ldi		TEMP_0,' '
		rcall	uart_send_byte

		load_p	Z,DATA_BUF+2
		ldi		TEMP_1,15

main_version_loop:
		ld		TEMP_0,Z+				; Startstring ausgeben
		rcall	uart_send_byte
		dec		TEMP_1
		brne	main_version_loop

		rcall	uart_send_newline

		rjmp	main

main_daten:
		load_p	Z,(2*text_temperatur)
		rcall	uart_send_text

		ld		TEMP_0,Y+				; Vorzeichen
		rcall	uart_send_byte

		ld		TEMP_0,Y+
		cpi		TEMP_0,'0'
		brne	main_loop_temp_no_0

		ldi		TEMP_0,' '

main_loop_temp_no_0:
		rcall	uart_send_byte
		ld		TEMP_0,Y+
		rcall	uart_send_byte

		ldi		TEMP_0,','
		rcall	uart_send_byte

		ld		TEMP_0,Y+
		rcall	uart_send_byte

		adiw	Y,1

		ld		TEMP_0,Y
		cpi		TEMP_0,0x55
		breq	main_loop_no_feuchte

		load_p	Z,(2*text_feuchte)
		rcall	uart_send_text

		ld		TEMP_0,Y+
		cpi		TEMP_0,'0'
		brne	main_loop_feuchte_no_0

		ldi		TEMP_0,' '

main_loop_feuchte_no_0:
		rcall	uart_send_byte
		ld		TEMP_0,Y+
		rcall	uart_send_byte

		ldi		TEMP_0,'%'
		rcall	uart_send_byte

		adiw	Y,1
		rjmp	main_loop_druck

main_loop_no_feuchte:
		adiw	Y,3

		load_p	Z,(2*text_grad)
		rcall	uart_send_text

main_loop_druck:
		rcall	check_sensor_druck
		brts	main_loop_no_druck

		sbiw	Y,6
		load_p	Z,(2*text_luftdruck_1)
		rcall	uart_send_text

		adiw	Y,1
		ld		TEMP_0,Y+
		rcall	uart_send_byte
		ld		TEMP_0,Y+
		rcall	uart_send_byte
		ld		TEMP_0,Y+
		rcall	uart_send_byte
		ld		TEMP_0,Y+
		rcall	uart_send_byte
	
		load_p	Z,(2*text_luftdruck_2)
		rcall	uart_send_text

		adiw	Y,1
		rjmp	main_loop_batt

main_loop_no_druck:


main_loop_batt:
		ld		TEMP_0,Y
		cpi		TEMP_0,0xA5				; Batterie
		breq	main_end				; ok

		load_p	Z,(2*text_batterie)
		rcall	uart_send_text
																	
main_end:		
		rcall	uart_send_newline

		rjmp	main

;*********************************************************************				 

.include "Subroutinen.inc"
.include "RFM12.inc"
.include "UART.inc"
.include "Tabellen.inc"

